@@ -6,13 +6,15 @@ import os
 import re
 import subprocess as sp
 import sys
+import time
 import warnings
 from time import sleep
 
-from options import use_gi
+from options import use_colord_gi
 
 try:
-	if not use_gi:
+	# XXX D-Bus API is more complete currently
+	if not use_colord_gi:
 		raise ImportError("")
 	from gi.repository import Colord
 	from gi.repository import Gio
@@ -22,14 +24,29 @@ except ImportError:
 else:
 	cancellable = Gio.Cancellable.new();
 
-if sys.platform not in ("darwin", "win32"):
-	from defaultpaths import xdg_data_home
+from util_dbus import DBusObject, DBusException, BUSTYPE_SYSTEM
 
 from util_os import which
 from util_str import safe_str, safe_unicode
 import localization as lang
 
-if not Colord or not hasattr(Colord, 'quirk_vendor_name'):
+if sys.platform not in ("darwin", "win32"):
+	from defaultpaths import xdg_data_home
+
+	if not Colord:
+		try:
+			Colord = DBusObject(BUSTYPE_SYSTEM, "org.freedesktop.ColorManager", 
+												"/org/freedesktop/ColorManager")
+		except DBusException, exception:
+			warnings.warn(safe_str(exception), Warning)
+
+
+# See colord/cd-client.c
+CD_CLIENT_IMPORT_DAEMON_TIMEOUT = 5000  # ms
+
+
+if (not Colord or isinstance(Colord, DBusObject) or
+	not hasattr(Colord, 'quirk_vendor_name')):
 	from config import get_data_path
 
 	# From colord/lib/colord/cd_quirk.c, cd_quirk_vendor_name
@@ -124,7 +141,7 @@ def device_connect(client, device_id):
 	return device
 
 
-def device_id_from_edid(edid, quirk=True, use_serial_32=True,
+def device_id_from_edid(edid, quirk=False, use_serial_32=True,
 						truncate_edid_strings=False,
 						omit_manufacturer=False, query=False):
 	""" Assemble device key from EDID """
@@ -136,17 +153,16 @@ def device_id_from_edid(edid, quirk=True, use_serial_32=True,
 		device_id = device_ids.get(edid["hash"])
 		if device_id:
 			return device_id
-		elif sys.platform not in ("darwin", "win32") and (query and
-														  which("colormgr")):
+		elif (sys.platform not in ("darwin", "win32") and query and
+			  isinstance(Colord, DBusObject)):
 			try:
-				device = find("device-by-property", ["OutputEdidMd5",
-													 edid["hash"]])
+				device = Device(find("device-by-property", ["OutputEdidMd5",
+															edid["hash"]]))
+				device_id = device.properties.get("DeviceId")
 			except CDError, exception:
 				warnings.warn(safe_str(exception), Warning)
 			else:
-				device_id = re.search(r":\s*(xrandr-[^\r\n]+)", device)
 				if device_id:
-					device_id = device_id.groups()[0]
 					device_ids[edid["hash"]] = device_id
 					return device_id
 	parts = ["xrandr"]
@@ -174,172 +190,194 @@ def device_id_from_edid(edid, quirk=True, use_serial_32=True,
 
 
 def find(what, search):
-	colormgr = which("colormgr")
-	if not colormgr:
-		raise CDError("colormgr helper program not found")
+	""" Find device or profile and return object path """
+	if not isinstance(Colord, DBusObject):
+		raise CDError("colord API not available")
 	if not isinstance(search, list):
 		search = [search]
-	args = ["find-%s" % what] + search
+	method_name = "_".join(part for part in what.split("-"))
 	try:
-		p = sp.Popen([safe_str(colormgr)] + args, stdout=sp.PIPE,
-					 stderr=sp.STDOUT)
-		stdout, stderr = p.communicate()
+		return getattr(Colord, "find_" + method_name)(*search)
 	except Exception, exception:
+		if hasattr(exception, "get_dbus_name"):
+			if exception.get_dbus_name() == "org.freedesktop.ColorManager.NotFound":
+				raise CDObjectNotFoundError(safe_str(exception))
+			else:
+				raise CDObjectQueryError(safe_str(exception))
 		raise CDError(safe_str(exception))
-	else:
-		errmsg = "Could not find %s for %s" % (what, search)
-		if p.returncode != 0:
-			raise CDObjectQueryError(stdout.strip() or errmsg)
-		result = stdout.strip()
-		if not result:
-			raise CDObjectNotFoundError(errmsg)
-	return result
 
 
 def get_default_profile(device_id):
 	"""
-	Get default profile filename for device
+	Get default profile for device
 	
 	"""
-	if not Colord:
-		colormgr = which("colormgr")
-		if not colormgr:
-			raise CDError("colormgr helper program not found")
-
-		# Find device object path
-		device = get_object_path(device_id, "device")
-		
-		# Get default profile
-		try:
-			p = sp.Popen([safe_str(colormgr), "device-get-default-profile",
-						  device],
-						 stdout=sp.PIPE, stderr=sp.STDOUT)
-			stdout, stderr = p.communicate()
-		except Exception, exception:
-			raise CDError(safe_str(exception))
-		else:
-			errmsg = "Couldn't get default profile for device %s" % device_id
-			if p.returncode != 0:
-				raise CDError(stdout.strip() or errmsg)
-			match = re.search(":\s*([^\r\n]+\.ic[cm])", stdout, re.I)
-			if match:
-				return safe_unicode(match.groups()[0])
-			else:
-				raise CDError(errmsg)
-		
-	client = client_connect()
-
-	# Connect to existing device
-	device = device_connect(client, device_id)
+	# Find device object path
+	device = Device(get_object_path(device_id, "device"))
 	
 	# Get default profile
-	profile = device.get_default_profile()
-	if not profile:
-		# No assigned profile
-		return
-
-	# Connect to profile
-	if not profile.connect_sync(cancellable):
-		raise CDError("Couldn't get default profile for device ID %r" % device_id)
-	filename = profile.get_filename()
-	if not isinstance(filename, unicode):
-		filename = filename.decode('UTF-8')
-	return filename
-
-
-def get_display_device_ids():
-	displays = []
-	colormgr = which("colormgr")
-	if not colormgr:
-		raise CDError("colormgr helper program not found")
 	try:
-		p = sp.Popen([safe_str(colormgr), "get-devices-by-kind", "display"],
-					 stdout=sp.PIPE, stderr=sp.STDOUT)
-		stdout, stderr = p.communicate()
+		properties = device.properties
 	except Exception, exception:
 		raise CDError(safe_str(exception))
 	else:
-		for line in stdout.splitlines():
-			item = line.split(":", 1)
-			if len(item) > 1 and item[1].strip().startswith("xrandr-"):
-				# Found device ID
-				displays.append(item[1].strip())
-	return displays
+		if properties.get("ProfilingInhibitors"):
+			return None
+		profiles = properties.get("Profiles")
+		if profiles:
+			return profiles[0]
+		else:
+			raise CDError("Couldn't get default profile for device ID %r" %
+						  device_id)
+
+
+def get_devices_by_kind(kind):
+	if not isinstance(Colord, DBusObject):
+		return []
+	return [Device(unicode(object_path, "UTF-8"))
+			for object_path in Colord.get_devices_by_kind(kind)]
+
+
+def get_display_devices():
+	return get_devices_by_kind("display")
+
+
+def get_display_device_ids():
+	return filter(None, (display.properties.get("DeviceId")
+						 for display in get_display_devices()))
 
 
 def get_object_path(search, object_type):
-	result = find(object_type, search)
-	if result:
-		result = result.splitlines()[0].split(":", 1)[-1].strip()
+	""" Get object path for profile or device ID """
+	result = find(object_type + "-by-id", search)
 	if not result:
 		raise CDObjectNotFoundError("Could not find object path for %s" % search)
 	return result
 
 
-def install_profile(device_id, profile, profile_installname=None,
-					timeout=20, logfn=None):
+def install_profile(device_id, profile,
+					timeout=CD_CLIENT_IMPORT_DAEMON_TIMEOUT / 1000.0,
+					logfn=None):
 	"""
 	Install profile for device
 	
-	profile_installname   filename of the installed profile (full path).
-						  The profile is copied to this location.
-						  If profile_installname is None, it defaults to
-						  ~/.local/share/icc/<profile basename>
 	timeout				  Time to allow for colord to pick up new profiles
 						  (recommended not below 2 secs)
 	
 	"""
 
+	profile_installname = os.path.join(xdg_data_home, 'icc',
+									   os.path.basename(profile.fileName))
+
+	profile_exists = os.path.isfile(profile_installname)
+	if (profile.fileName != profile_installname and
+		profile_exists):
+		if logfn:
+			logfn(u"About to overwrite existing", profile_installname)
+		profile.fileName = None
+
 	if profile.ID == "\0" * 16:
 		profile.calculateID()
+		profile.fileName = None
 	profile_id = "icc-" + hexlify(profile.ID)
 
 	# Write profile to destination
-	if not profile_installname:
-		profile_installname = os.path.join(xdg_data_home, 'icc',
-										   os.path.basename(profile.fileName))
 	profile_installdir = os.path.dirname(profile_installname)
 	if not os.path.isdir(profile_installdir):
 		os.makedirs(profile_installdir)
-	profile.write(profile_installname)
-	
-	if isinstance(profile_installname, unicode):
-		profile_installname = profile_installname.encode('UTF-8')
+	# colormgr seems to have a bug where the first attempt at importing a
+	# specific profile can time out. This seems to be work-aroundable by
+	# writing the profile ourself first, and then importing.
+	if not profile.fileName or not profile_exists:
+		if logfn:
+			logfn(u"Writing", profile_installname)
+		profile.fileName = profile_installname
+		profile.write()
 
-	if Colord:
+	cdprofile = None
+
+	if Colord and not isinstance(Colord, DBusObject):
 		client = client_connect()
 	else:
+		# Query colord for profile
+		try:
+			cdprofile = get_object_path(profile_id, "profile")
+		except CDObjectQueryError, exception:
+			# Profile not found
+			pass
+
 		colormgr = which("colormgr")
 		if not colormgr:
 			raise CDError("colormgr helper program not found")
 
-		profile = None
+		from worker import printcmdline
 
-	# Query colord for newly added profile
-	for i in xrange(int(timeout / 1.0)):
-		try:
-			if Colord:
-				profile = client.find_profile_sync(profile_id, cancellable)
-			else:
-				profile = get_object_path(profile_id, "profile")
-		except CDObjectQueryError, exception:
-			# Profile not found
-			pass
-		if profile:
-			break
-		# Give colord time to pick up the profile
-		sleep(1)
+		cmd = safe_str(colormgr)
 
-	if not profile:
-		raise CDTimeout("Querying for profile %r returned no result for %s secs" %
-						(profile_id, timeout))
+		if not cdprofile:
+			# Import profile
+
+			if logfn:
+				logfn("-" * 80)
+				logfn(lang.getstr("commandline"))
+
+			args = [cmd, "import-profile", safe_str(profile.fileName)]
+			printcmdline(args[0], args[1:], fn=logfn)
+			if logfn:
+				logfn("")
+
+			ts = time.time()
+
+			maxtries = 3
+
+			for n in xrange(1, maxtries + 1):
+				if logfn:
+					logfn("Trying to import profile, attempt %i..." % n)
+				try:
+					p = sp.Popen(args, stdout=sp.PIPE, stderr=sp.STDOUT)
+					stdout, stderr = p.communicate()
+				except Exception, exception:
+					raise CDError(safe_str(exception))
+				if logfn and stdout.strip():
+					logfn(stdout.strip())
+				if p.returncode == 0 or os.path.isfile(profile_installname):
+					if logfn:
+						logfn("...ok")
+					break
+				elif logfn:
+					logfn("...failed!")
+
+			if p.returncode != 0 and not os.path.isfile(profile_installname):
+				raise CDTimeout("Trying to import profile '%s' failed after "
+								"%i tries." % (profile.fileName, n))
+
+	if not cdprofile:
+		# Query colord for newly added profile
+		for i in xrange(int(timeout / 1.0)):
+			try:
+				if Colord and not isinstance(Colord, DBusObject):
+					cdprofile = client.find_profile_sync(profile_id,
+														 cancellable)
+				else:
+					cdprofile = get_object_path(profile_id, "profile")
+			except CDObjectQueryError, exception:
+				# Profile not found
+				pass
+			if cdprofile:
+				break
+			# Give colord time to pick up the profile
+			sleep(1)
+
+		if not cdprofile:
+			raise CDTimeout("Querying for profile %r returned no result for %s "
+							"secs" % (profile_id, timeout))
 
 	errmsg = "Could not make profile %s default for device %s" % (profile_id,
 																  device_id)
 
-	if Colord:
+	if Colord and not isinstance(Colord, DBusObject):
 		# Connect to profile
-		if not profile.connect_sync(cancellable):
+		if not cdprofile.connect_sync(cancellable):
 			raise CDError("Could not connect to profile")
 
 		# Connect to existing device
@@ -347,13 +385,14 @@ def install_profile(device_id, profile, profile_installname=None,
 
 		# Add profile to device
 		try:
-			device.add_profile_sync(Colord.DeviceRelation.HARD, profile, cancellable)
+			device.add_profile_sync(Colord.DeviceRelation.HARD, cdprofile,
+									cancellable)
 		except Exception, exception:
 			# Profile may already have been added
 			warnings.warn(safe_str(exception), Warning)
 
 		# Make profile default for device
-		if not device.make_profile_default_sync(profile, cancellable):
+		if not device.make_profile_default_sync(cdprofile, cancellable):
 			raise CDError(errmsg)
 	else:
 		# Find device object path
@@ -363,13 +402,9 @@ def install_profile(device_id, profile, profile_installname=None,
 			logfn("-" * 80)
 			logfn(lang.getstr("commandline"))
 
-		from worker import printcmdline
-
-		cmd = safe_str(colormgr)
-
 		# Add profile to device
 		# (Ignore returncode as profile may already have been added)
-		args = [cmd, "device-add-profile", device, profile]
+		args = [cmd, "device-add-profile", device, cdprofile]
 		printcmdline(args[0], args[1:], fn=logfn)
 		if logfn:
 			logfn("")
@@ -386,7 +421,7 @@ def install_profile(device_id, profile, profile_installname=None,
 			logfn(lang.getstr("commandline"))
 
 		# Make profile default for device
-		args = [cmd, "device-make-profile-default", device, profile]
+		args = [cmd, "device-make-profile-default", device, cdprofile]
 		printcmdline(args[0], args[1:], fn=logfn)
 		if logfn:
 			logfn("")
@@ -403,7 +438,8 @@ def install_profile(device_id, profile, profile_installname=None,
 
 
 def quirk_manufacturer(manufacturer):
-	if Colord and hasattr(Colord, 'quirk_vendor_name'):
+	if (Colord and not isinstance(Colord, DBusObject) and
+		hasattr(Colord, 'quirk_vendor_name')):
 		return Colord.quirk_vendor_name(manufacturer)
 
 	# Correct some company names
@@ -422,6 +458,44 @@ def quirk_manufacturer(manufacturer):
 	return manufacturer
 
 
+class Object(DBusObject):
+
+	def __init__(self, object_path, object_type):
+		try:
+			DBusObject.__init__(self, BUSTYPE_SYSTEM,
+								"org.freedesktop.ColorManager", object_path,
+								object_type)
+		except DBusException, exception:
+			raise CDError(safe_str(exception))
+		self._object_type = object_type
+
+	_properties = DBusObject.properties
+
+	@property
+	def properties(self):
+		try:
+			properties = {}
+			for key, value in self._properties.iteritems():
+				if key == "Profiles":
+					value = [Profile(object_path) for object_path in value]
+				properties[key] = value
+			return properties
+		except DBusException, exception:
+			raise CDError(safe_str(exception))
+
+
+class Device(Object):
+
+	def __init__(self, object_path):
+		Object.__init__(self, object_path, "Device")
+
+
+class Profile(Object):
+
+	def __init__(self, object_path):
+		Object.__init__(self, object_path, "Profile")
+
+
 class CDError(Exception):
 	pass
 
