@@ -454,6 +454,8 @@ class Xicclu(WorkerBase):
 				 show_actual_if_clipped=False, input_encoding=None,
 				 output_encoding=None, convert_video_rgb_to_clut65=False,
 				 verbose=1):
+		if not profile:
+			raise Error("Xicclu: Profile is %r" % profile)
 		WorkerBase.__init__(self)
 		self.scale = scale
 		self.convert_video_rgb_to_clut65 = convert_video_rgb_to_clut65
@@ -470,18 +472,20 @@ class Xicclu(WorkerBase):
 			else:
 				profile = ICCP.ICCProfile(profile)
 		is_profile = isinstance(profile, ICCP.ICCProfile)
-		if is_profile:
+		if (is_profile and profile.version >= 4 and
+			not profile.convert_iccv4_tags_to_iccv2()):
+			raise Error("\n".join([lang.getstr("profile.iccv4.unsupported"),
+											   profile.getDescription()]))
+		if not profile.fileName or not os.path.isfile(profile.fileName):
 			if profile.fileName:
 				prefix = os.path.basename(profile.fileName)
-			else:
+			elif is_profile:
 				prefix = make_filename_safe(profile.getDescription(),
 											concat=False) + profile_ext
+			else:
+				# CGATS (.cal)
+				prefix = "cal"
 			prefix += "-"
-			if (profile.version >= 4 and
-				not profile.convert_iccv4_tags_to_iccv2()):
-				raise Error("\n".join([lang.getstr("profile.iccv4.unsupported"),
-												   profile.getDescription()]))
-		if not profile.fileName or not os.path.isfile(profile.fileName):
 			if not cwd:
 				cwd = self.create_tempdir()
 				if isinstance(cwd, Exception):
@@ -523,7 +527,8 @@ class Xicclu(WorkerBase):
 					input_encoding = "n"
 				if not output_encoding:
 					output_encoding = "n"
-				args += ["-e" + input_encoding, "-E" + output_encoding]
+				args += ["-e" + safe_str(input_encoding),
+						 "-E" + safe_str(output_encoding)]
 		args.append("-f" + direction)
 		self.output_scale = 1.0
 		if is_profile:
