@@ -7,7 +7,7 @@ Interactive display calibration UI
 """
 
 from __future__ import with_statement
-from time import strftime
+from time import sleep, strftime
 import os
 import re
 import sys
@@ -95,6 +95,7 @@ class DisplayUniformityFrame(BaseFrame):
 		self.disable_buttons()
 		
 		self.keyhandler = keyhandler
+		self.id_to_keycode = {}
 		if sys.platform == "darwin":
 			# Use an accelerator table for tab, space, 0-9, A-Z, numpad,
 			# navigation keys and processing keys
@@ -104,9 +105,8 @@ class DisplayUniformityFrame(BaseFrame):
 			keycodes.extend(numpad_keycodes)
 			keycodes.extend(nav_keycodes)
 			keycodes.extend(processing_keycodes)
-			self.id_to_keycode = {}
 			for keycode in keycodes:
-				self.id_to_keycode[wx.NewId()] = keycode
+				self.id_to_keycode[wx.Window.NewControlId()] = keycode
 			accels = []
 			for id, keycode in self.id_to_keycode.iteritems():
 				self.Bind(wx.EVT_MENU, self.key_handler, id=id)
@@ -149,6 +149,13 @@ class DisplayUniformityFrame(BaseFrame):
 	def OnDestroy(self, event):
 		self.stop_timer()
 		del self.timer
+		if hasattr(wx.Window, "UnreserveControlId"):
+			for id in self.id_to_keycode.iterkeys():
+				if id < 0:
+					try:
+						wx.Window.UnreserveControlId(id)
+					except wx.wxAssertionError, exception:
+						safe_print(exception)
 		
 	def OnMove(self, event):
 		pass
@@ -242,6 +249,7 @@ class DisplayUniformityFrame(BaseFrame):
 	def measure(self, event=None):
 		if event:
 			self.index = event.GetEventObject().index
+			safe_print("%s: Uniformity grid index %i" % (appname, self.index))
 			self.is_measuring = True
 			self.results[self.index] = []
 			self.labels[self.index].SetLabel("")
@@ -251,6 +259,9 @@ class DisplayUniformityFrame(BaseFrame):
 		self.panels[self.index].SetBackgroundColour(self.colors[len(self.results[self.index])])
 		self.panels[self.index].Refresh()
 		self.panels[self.index].Update()
+		safe_print("%s: About to measure uniformity grid index %i @%i%%" %
+				   (appname, self.index,
+					self.colors[len(self.results[self.index])].red / 2.55))
 		# Use a delay to allow for TFT lag
 		wx.CallLater(200, self.safe_send, " ")
 
@@ -278,11 +289,13 @@ class DisplayUniformityFrame(BaseFrame):
 							   txt, re.I)
 				self.results[self.index][-1]["C%sT" % locus[0]] = int(CT.groups()[0])
 		if "key to take a reading" in txt and not self.last_error:
+			safe_print("%s: Got 'key to take a reading'" % appname)
 			if not self.is_measuring:
 				self.enable_buttons()
 				return
 			if len(self.results[self.index]) < len(self.colors):
 				# Take readings at 5 different brightness levels per swatch
+				safe_print("%s: About to take next reading" % appname)
 				self.measure()
 			else:
 				self.is_measuring = False
@@ -374,7 +387,15 @@ class DisplayUniformityFrame(BaseFrame):
 	
 	def safe_send(self, bytes):
 		if self.has_worker_subprocess() and not self.worker.subprocess_abort:
-			self.worker.safe_send(bytes)
+			if not self.worker.instrument_on_screen:
+				if not getattr(self, "wait_for_instrument_on_screen", False):
+					self.wait_for_instrument_on_screen = True
+					safe_print("%s: Waiting for instrument to be placed on screen" %
+							   appname)
+				wx.CallLater(200, self.safe_send, bytes)
+			else:
+				self.wait_for_instrument_on_screen = False
+				self.worker.safe_send(bytes)
 	
 	def show_cursor(self):
 		cursor = wx.StockCursor(wx.CURSOR_ARROW)
